package org.liquidengine.legui.component.misc.listener.textarea;

import org.joml.Vector2f;
import org.joml.Vector4f;
import org.liquidengine.legui.component.Component;
import org.liquidengine.legui.component.ScrollablePanel;
import org.liquidengine.legui.component.TextArea;
import org.liquidengine.legui.component.TextAreaField;
import org.liquidengine.legui.event.ScrollEvent;
import org.liquidengine.legui.input.Mouse;
import org.liquidengine.legui.listener.ScrollEventListener;
import org.liquidengine.legui.system.handler.SehUtil;

import java.util.ArrayList;

import static org.liquidengine.legui.component.misc.listener.EventUtils.hasViewportsInAboveLayersUnderCursor;
import static org.liquidengine.legui.component.misc.listener.scrollbar.ScrollBarHelper.updateScrollBarValue;
import static org.liquidengine.legui.style.util.StyleUtilities.getPadding;

/**
 * Created by ShchAlexander on 23.07.2017.
 */
public class TextAreaViewportScrollListener implements ScrollEventListener {


    /**
     * Used to handle specific event.
     *
     * @param event event to handle.
     */
    @Override
    public void process(ScrollEvent event) {
        Vector2f cursorPosition = Mouse.getCursorPosition();
        Component targetComponent = event.getTargetComponent();

        if (hasViewportsInAboveLayersUnderCursor(targetComponent, cursorPosition)) return;

        ArrayList<Component> targetList = new ArrayList<>();
        SehUtil.recursiveTargetComponentListSearch(Mouse.getCursorPosition(), event.getTargetComponent(), targetList);
        for (Component component : targetList) {
            if ((component instanceof TextArea) || (component instanceof ScrollablePanel)) {
                return;
            }
        }

        TextArea textArea = (TextArea) event.getTargetComponent().getParent();

        TextAreaField textAreaField = textArea.getTextAreaField();
        Vector4f padding = getPadding(textAreaField, textAreaField.getStyle());
        float maxTextWidth = Math.max(
            textAreaField.getMaxTextWidth() + padding.x + padding.z,
            textArea.getViewportSize().x
        );
        float maxTextHeight = Math.max(
            textAreaField.getMaxTextHeight() + padding.y + padding.w,
            textArea.getViewportSize().y
        );
        textAreaField.setSize(maxTextWidth, maxTextHeight);

        if (Math.abs(event.getYoffset()) > 0)
            updateScrollBarValue(event.getYoffset(), event.getContext(), event.getFrame(), textArea.getVerticalScrollBar());
        if (Math.abs(event.getXoffset()) > 0)
            updateScrollBarValue(event.getXoffset(), event.getContext(), event.getFrame(), textArea.getHorizontalScrollBar());
    }
}
